@@ -4,6 +4,7 @@ from __future__ import annotations
 
 import asyncio
 import logging
+import warnings
 from collections.abc import Generator
 from contextlib import contextmanager
 from typing import TYPE_CHECKING, Any, Final, Iterable, final
@@ -36,6 +37,14 @@ _int = int
 _str = str
 
 
+def _dict_subset(super_dict: dict[Any, bytes], sub_dict: dict[Any, bytes]) -> bool:
+    """Return True if sub_dict is a subset of super_dict."""
+    for key, sub_value in sub_dict.items():
+        if (super_value := super_dict.get(key)) is None or super_value != sub_value:
+            return False
+    return True
+
+
 class BaseHaScanner:
     """Base class for high availability BLE scanners."""
 
@@ -143,8 +152,7 @@ class BaseHaScanner:
         is triggered.
         """
         if self._async_watchdog_triggered():
-            _LOGGER.log(
-                logging.WARNING if self.scanning else logging.DEBUG,
+            _LOGGER.debug(
                 (
                     "%s: Bluetooth scanner has gone quiet for %ss, check logs on the"
                     " scanner device for more information"
@@ -285,29 +293,40 @@ class BaseHaRemoteScanner(BaseHaScanner):
         return DiscoveredDeviceAdvertisementData(
             self.connectable,
             self._expire_seconds,
-            self._discovered_device_advertisement_datas,
-            self._discovered_device_timestamps,
+            self._build_discovered_device_advertisement_datas(),
+            self._build_discovered_device_timestamps(),
         )
 
-    @property
-    def _discovered_device_advertisement_datas(
-        self,
-    ) -> dict[str, tuple[BLEDevice, AdvertisementData]]:
-        """Return a list of discovered devices and advertisement data."""
-        return {
-            address: (
-                service_info.device,
-                service_info.advertisement,
-            )
-            for address, service_info in self._previous_service_info.items()
-        }
-
     @property
     def _discovered_device_timestamps(self) -> dict[str, float]:
         """Return a dict of discovered device timestamps."""
+        warnings.warn(
+            "BaseHaRemoteScanner._discovered_device_timestamps is deprecated "
+            "and will be removed in a future version of habluetooth, use "
+            "BaseHaRemoteScanner.discovered_device_timestamps instead",
+            FutureWarning,
+            stacklevel=2,
+        )
+        return self._build_discovered_device_timestamps()
+
+    @property
+    def discovered_device_timestamps(self) -> dict[str, float]:
+        """Return a dict of discovered device timestamps."""
+        return self._build_discovered_device_timestamps()
+
+    def _build_discovered_device_advertisement_datas(
+        self,
+    ) -> dict[str, tuple[BLEDevice, AdvertisementData]]:
+        """Return a list of discovered devices and advertisement data."""
         return {
-            address: service_info.time
-            for address, service_info in self._previous_service_info.items()
+            address: (info.device, info._advertisement_internal())
+            for address, info in self._previous_service_info.items()
+        }
+
+    def _build_discovered_device_timestamps(self) -> dict[str, float]:
+        """Return a dict of discovered device timestamps."""
+        return {
+            address: info.time for address, info in self._previous_service_info.items()
         }
 
     def _cancel_expire_devices(self) -> None:
@@ -348,8 +367,8 @@ class BaseHaRemoteScanner(BaseHaScanner):
         now = monotonic_time_coarse()
         expired = [
             address
-            for address, service_info in self._previous_service_info.items()
-            if now - service_info.time > self._expire_seconds
+            for address, info in self._previous_service_info.items()
+            if now - info.time > self._expire_seconds
         ]
         for address in expired:
             del self._previous_service_info[address]
@@ -357,18 +376,15 @@ class BaseHaRemoteScanner(BaseHaScanner):
     @property
     def discovered_devices(self) -> list[BLEDevice]:
         """Return a list of discovered devices."""
-        service_infos = self._previous_service_info.values()
-        return [
-            device_advertisement_data.device
-            for device_advertisement_data in service_infos
-        ]
+        infos = self._previous_service_info.values()
+        return [device_advertisement_data.device for device_advertisement_data in infos]
 
     @property
     def discovered_devices_and_advertisement_data(
         self,
     ) -> dict[str, tuple[BLEDevice, AdvertisementData]]:
         """Return a list of discovered devices and advertisement data."""
-        return self._discovered_device_advertisement_datas
+        return self._build_discovered_device_advertisement_datas()
 
     @property
     def discovered_addresses(self) -> Iterable[str]:
@@ -398,28 +414,28 @@ class BaseHaRemoteScanner(BaseHaScanner):
         """Call the registered callback."""
         self.scanning = not self._connecting
         self._last_detection = advertisement_monotonic_time
-        service_info = BluetoothServiceInfoBleak.__new__(BluetoothServiceInfoBleak)
+        info = BluetoothServiceInfoBleak.__new__(BluetoothServiceInfoBleak)
 
-        if (prev_service_info := self._previous_service_info.get(address)) is None:
+        if (prev_info := self._previous_service_info.get(address)) is None:
             # We expect this is the rare case and since py3.11+ has
             # near zero cost try on success, and we can avoid .get()
             # which is slower than [] we use the try/except pattern.
-            service_info.device = BLEDevice(
+            info.device = BLEDevice(
                 address,
                 local_name,
                 {**self._details, **details},
                 rssi,  # deprecated, will be removed in newer bleak
             )
-            service_info.manufacturer_data = manufacturer_data
-            service_info.service_data = service_data
-            service_info.service_uuids = service_uuids
-            service_info.name = local_name or address
+            info.manufacturer_data = manufacturer_data
+            info.service_data = service_data
+            info.service_uuids = service_uuids
+            info.name = local_name or address
         else:
             # Merge the new data with the old data
             # to function the same as BlueZ which
             # merges the dicts on PropertiesChanged
-            service_info.device = prev_service_info.device
-            prev_name = prev_service_info.device.name
+            info.device = prev_info.device
+            prev_name = prev_info.device.name
             #
             # Bleak updates the BLEDevice via create_or_update_device.
             # We need to do the same to ensure integrations that already
@@ -428,67 +444,78 @@ class BaseHaRemoteScanner(BaseHaScanner):
             #
             # https://github.com/hbldh/bleak/blob/222618b7747f0467dbb32bd3679f8cfaa19b1668/bleak/backends/scanner.py#L203
             #
-            prev_details: dict[str, Any] = service_info.device.details
+            prev_details: dict[str, Any] = info.device.details
             prev_details.update(details)
             # _rssi is deprecated, will be removed in newer bleak
             # pylint: disable-next=protected-access
-            service_info.device._rssi = rssi
+            info.device._rssi = rssi
             has_local_name = bool(local_name)
-            if prev_name and (not has_local_name or len(prev_name) > len(local_name)):  # type: ignore[arg-type]
-                service_info.name = prev_name
+            if prev_name and (
+                not has_local_name
+                or (prev_name is not local_name and len(prev_name) > len(local_name))  # type: ignore[arg-type]
+            ):
+                info.name = prev_name
             else:
-                service_info.device.name = local_name
-                service_info.name = local_name if has_local_name else address  # type: ignore[assignment]
+                info.device.name = local_name
+                info.name = local_name if has_local_name else address  # type: ignore[assignment]
 
             has_service_uuids = bool(service_uuids)
-            if has_service_uuids and service_uuids != prev_service_info.service_uuids:
-                service_info.service_uuids = list(
-                    {*service_uuids, *prev_service_info.service_uuids}
-                )
+            if (
+                has_service_uuids
+                and service_uuids is not prev_info.service_uuids
+                and service_uuids != prev_info.service_uuids
+            ):
+                info.service_uuids = list({*service_uuids, *prev_info.service_uuids})
             elif not has_service_uuids:
-                service_info.service_uuids = prev_service_info.service_uuids
+                info.service_uuids = prev_info.service_uuids
             else:
-                service_info.service_uuids = service_uuids
+                info.service_uuids = service_uuids
 
             has_service_data = bool(service_data)
-            if has_service_data and service_data != prev_service_info.service_data:
-                service_info.service_data = {
-                    **prev_service_info.service_data,
-                    **service_data,
-                }
+            if has_service_data and service_data is not prev_info.service_data:
+                if _dict_subset(prev_info.service_data, service_data):
+                    info.service_data = prev_info.service_data
+                else:
+                    info.service_data = {
+                        **prev_info.service_data,
+                        **service_data,
+                    }
             elif not has_service_data:
-                service_info.service_data = prev_service_info.service_data
+                info.service_data = prev_info.service_data
             else:
-                service_info.service_data = service_data
+                info.service_data = service_data
 
             has_manufacturer_data = bool(manufacturer_data)
             if (
                 has_manufacturer_data
-                and manufacturer_data != prev_service_info.manufacturer_data
+                and manufacturer_data is not prev_info.manufacturer_data
             ):
-                service_info.manufacturer_data = {
-                    **prev_service_info.manufacturer_data,
-                    **manufacturer_data,
-                }
+                if _dict_subset(prev_info.manufacturer_data, manufacturer_data):
+                    info.manufacturer_data = prev_info.manufacturer_data
+                else:
+                    info.manufacturer_data = {
+                        **prev_info.manufacturer_data,
+                        **manufacturer_data,
+                    }
             elif not has_manufacturer_data:
-                service_info.manufacturer_data = prev_service_info.manufacturer_data
+                info.manufacturer_data = prev_info.manufacturer_data
             else:
-                service_info.manufacturer_data = manufacturer_data
+                info.manufacturer_data = manufacturer_data
 
-        service_info.address = address
-        service_info.rssi = rssi
-        service_info.source = self.source
-        service_info._advertisement = None
-        service_info.connectable = self.connectable
-        service_info.time = advertisement_monotonic_time
-        service_info.tx_power = tx_power
-        self._previous_service_info[address] = service_info
-        self._manager.scanner_adv_received(service_info)
+        info.address = address
+        info.rssi = rssi
+        info.source = self.source
+        info._advertisement = None
+        info.connectable = self.connectable
+        info.time = advertisement_monotonic_time
+        info.tx_power = tx_power
+        self._previous_service_info[address] = info
+        self._manager.scanner_adv_received(info)
 
     async def async_diagnostics(self) -> dict[str, Any]:
         """Return diagnostic information about the scanner."""
         now = monotonic_time_coarse()
-        discovered_device_timestamps = self._discovered_device_timestamps
+        discovered_device_timestamps = self._build_discovered_device_timestamps()
         return await super().async_diagnostics() | {
             "discovered_device_timestamps": discovered_device_timestamps,
             "time_since_last_device_detection": {
