@@ -23,6 +23,7 @@ _BluetoothServiceInfoBleakSelfT = TypeVar(
     "_BluetoothServiceInfoBleakSelfT", bound="BluetoothServiceInfoBleak"
 )
 SOURCE_LOCAL: Final = "local"
+TUPLE_NEW: Final = tuple.__new__
 
 _float = float  # avoid cython conversion since we always want a pyfloat
 _str = str  # avoid cython conversion since we always want a pystr
@@ -208,11 +209,14 @@ class BluetoothServiceInfoBleak(BluetoothServiceInfo):
             f"tx_power={self.tx_power}>"
         )
 
-    @property
-    def advertisement(self) -> AdvertisementData:
-        """Get the advertisement data."""
+    def _advertisement_internal(self) -> AdvertisementData:
+        """
+        Get the advertisement data.
+
+        Internal method only to be used by this library.
+        """
         if self._advertisement is None:
-            self._advertisement = tuple.__new__(
+            self._advertisement = TUPLE_NEW(
                 AdvertisementData,
                 (
                     None if self.name == "" or self.name == self.address else self.name,
@@ -226,6 +230,11 @@ class BluetoothServiceInfoBleak(BluetoothServiceInfo):
             )
         return self._advertisement
 
+    @property
+    def advertisement(self) -> AdvertisementData:
+        """Get the advertisement data."""
+        return self._advertisement_internal()
+
     def as_dict(self) -> dict[str, Any]:
         """
         Return as dict.
